# -*- coding: iso-8859-1 -*-
"""
  2005-09-10, Hans Maerki, License LGPL
  
  Doubleclicking this file will loop over all
  image files in the current folder and create
  image-files and index.zulu_content.html.
  
  Installation:
    Run the following two installers to install python, the
    imaging library and the pdf generator.
    http://www.python.org/ftp/python/2.2.2/Python-2.2.2.exe
    It is a good idea to accept the installer's default settings.
  
    Now move this script into the folder with the images and
    double click it.
    If there is an error - you have about half a millisecond to
    see it. If this is to quick, open a dos-window, cd to the
    directory with your image-files and type "alpuma.py". Now
    you will see the error message.
  
   Pseudocode:
     for each *.jpg|*.gif|*.png
       convert image
       Get size of image
       if 'index.zulu_content.html' does not exist: create it from template.
       replace in 'index.zulu_content.html'
         href="bilder/06.jpg"
            <img src="bilder/06k.jpg" alt="Grossformat" border="0" width="200" height="150">
         if not found
            Add template at begin of page
  
  Kurzanleitung:
    Ordnerstruktur bereitstellen:
      base/alpuma_v2.0.0.py       (von Hans)
      base/alpuma_config.txt      (von Hans)
      base/orig/IMAGE123.JPG      (Hier die Bilder hineinkopieren
    'base/alpuma_v2.0.0.py' doppelklicken. Es entstehen folgende Files:
      'base/images/IMAGE123.JPG'
      'base/thumbs/IMAGE123.JPG'
      'base/index.zulu_content.html'
  
  Features:
    Imageverarbeitung:
      Ordner werden kreiert, falls noetig.
      Die Datei 'index.zulu_content.html' wird kreiert, falls noetig.
      Images werden nur dann neu geschrieben, falls sie inhaltlich geaendert haben.
      Die Breite des Images wird angepasst. Falls zu hoch, wird anhand der
      maximalen Hoehe skaliert.
  
    HTML aktualisieren:
      Falls 'html_filename' fehlt in 'alpuma_config.txt', wird kein
      HTML-File bearbeitet.
      width/height wird eingesetzt, falls nicht vorhanden.
      Entry fr image wird eingesetzt, falls nicht vorhanden.
      Falls ein File 'thumbs/IMAGE123_alpuma_override.JPG' existiert, wird
      auf dieses File referenziert statt auf 'thumbs/IMAGE123.JPG'.
      Falls ein File 'images/IMAGE123_alpuma_override.JPG' existiert und statt
      'images/IMAGE123.JPG' verwendet werden soll, so muss der neue Filename
      manuell im html-File nachgetragen werden.

    Annotations:
      In 'alpuma_config.txt' kann eine Annotation definiert werden. Dies
      ist zum Beispiel ein Copyright-Vermerk " Aldo Mustardo".
      Diese Annotation wird auf alle Bilder plaziert, auf welcher der
      Text Platz hat.

  History:
    2003-04-03, Hans Maerki. Python rules!
    2003-04-28, v2.0.0, Hans Maerki. Many new features.
    2003-04-28, v2.0.1, Hans Maerki. Minor bugfixes.
    2003-05-24, v2.0.2, Hans Maerki. Figaros Feature Request: Never enlarge images.
    2003-09-28, v2.0.3, Hans Maerki. 'Thumbs.db' wird ignoriert.
    2003-11-01, v2.0.4, Hans Maerki. Quality of the compression choosable now.
    2003-10-14, v2.0.5, Hans Maerki. Tested with Python 2.3
    2005-05-22, v2.0.6, Hans Maerki. Minor bugfixes.
    2005-09-10, v2.0.7, Hans Maerki. Refactoring - better structured. Added comments.
                                     Added Annotations.
"""

import os, string, filecmp, shutil
import PIL.Image
import PIL.ImageFileIO

config = {
  'html_filename' : None
}

def go():
  """Album assembler"""

  execfile("alpuma_config.txt", config)
  if config.get('alpuma_config_version', None) != '1.0.0':
    raise Exception("This Alpuma-Version is too old for this 'alpuma_config.txt'")

  # Open the content-html file. If any or needed...
  strFilenameHTML = config.get('html_filename')
  strContent = config.get('html_template_file')
  if strFilenameHTML != None:
    try:
      fileContent = open(config.get('html_filename'), 'r')
      strContent = fileContent.read()
      fileContent.close()
    except IOError, e:
      print "Template '%s' not found: Use template from 'alpuma_config.txt'." % config.get('html_filename')

  # Loop over all files in the images directory
  for (strFileRoot, strFileExt) in getFiles(config.get('input_path')):
    strFilename = strFileRoot + strFileExt
    dictParams = {'file':strFilename, 'fileroot':strFileRoot, 'fileext':strFileExt}
    print "Processing %s ..." % strFilename

    # Convert the images
    for conversion in config.get('conversions'):
      if not os.path.exists(conversion['output_path']):
        print "  Create folder %s ..." % conversion['output_path']
        os.mkdir(conversion['output_path'])
      iWidth, iHeight = convert_image(strFilename, conversion)

    if strFilenameHTML != None:
      # Update the HTML-File if needed
      strContent = updateContent(strContent, strFilename, dictParams)

  if strFilenameHTML != None:
    print "  Write %s ..." % strFilenameHTML
    fileContent = open(strFilenameHTML, 'w')
    fileContent.write(strContent)
    fileContent.close()

def getFiles(strPath):
  # Loop over all files in the images directory
  files = os.listdir(strPath)
  files.sort()
  files.reverse()
  listFiles = []
  for strFilename in files:
    if strFilename.lower() != "thumbs.db":
      listFiles.append(os.path.splitext(strFilename))
  return listFiles

def updateContent(strContent, strFilename, dictParams):
  strContentLower = string.lower(strContent)
  iStart = 0
  while 1:
    iStart = string.find(strContentLower, '<img', iStart)
    iEnd = string.find(strContentLower, '>', iStart+1)
    if (iStart==-1) or (iEnd==-1):
      # Entry not found: Add new entry
      print "  Entry not found: Add new entry!"
      return updateContent(addEntry(strContent, dictParams), strFilename, dictParams)
    for strFullPath in config.get('html_img_files'):
      strFullPath = getTemplate(strFullPath, dictParams)
      iPathStart = string.find(strContentLower, string.lower(strFullPath), iStart, iEnd)
      if iPathStart != -1:
        # We found the entry we are looking for. Update it
        # Get the first image in the list
        for strFullImagePath in config.get('html_img_files'):
          strFullImagePath = getTemplate(strFullImagePath, dictParams)
          if os.path.exists(strFullImagePath):
            return updateEntryInContent(strContent, strFullPath, strFullImagePath, strFilename, dictParams, iStart, iEnd)
    iStart = iEnd

def updateEntryInContent(strContent, strFullPath, strFullImagePath, strFilename, dictParams, iStart, iEnd):
  strEntry = strContent[iStart:iEnd]
  if strFullPath!=strFullImagePath:
    strEntry = string.replace(strEntry, strFullPath, strFullImagePath)
    print "  Renaming HTML '%s' -> '%s'." % (strFullPath, strFullImagePath)
  if string.find(strFullImagePath, strFilename) == -1:
    # Replace path if needed (case 'thumbs/{fileroot}_alpuma_override{fileext}')
    print "  Overriding HTML '%s'." % strFullImagePath
  strEntry = updateEntry(strEntry, strFullImagePath, dictParams)
  return strContent[:iStart] + strEntry + strContent[iEnd:]

def updateEntry(strEntry, strFullPath, dictParams):
  fp = open(strFullPath, "rb")
  image = PIL.Image.open(PIL.ImageFileIO.ImageFileIO(fp))
  image.load() # make sure we can read the raster data
  iWidth, iHeight = image.size
  strEntry = replaceSize(strEntry, "width=", iWidth)
  strEntry = replaceSize(strEntry, "height=", iHeight)
  return strEntry

def addEntry(strContent, dictParams):
  if string.find(strContent, '<!--AlpumaInsert-->') >= 0:
    strLeft, strRight = string.split(strContent, '<!--AlpumaInsert-->', 1)
  else:
    strLeft = ''
    strRight = strContent
  strEntry = getTemplate(config.get('html_template_image'), dictParams)
  return strLeft + '<!--AlpumaInsert-->' + strEntry + strRight
    
def getTemplate(strTemplate, dictParams):
  for (strTag, strValue) in dictParams.items():
    strTemplate = string.replace(strTemplate, '{%s}' % strTag, str(strValue))
  return strTemplate

def replaceSize(strEntry, strPattern, iValue):
  try:
    strStart, strValue = string.split(strEntry, strPattern+'"', 1)
    strValue, strEnd = string.split(strValue, '"', 1)
  except ValueError:
    # strPattern not found. Add it at the end.
    return '%s %s"%d"' % (strEntry, strPattern, iValue)
  # Pattern found. Update the size.
  return '%s%s"%d"%s' % (strStart, strPattern, iValue, strEnd)
  
def convert_image(strFilename, conversion):
  # Get default/configuration values
  iQuality = conversion.get('quality', 100) # Default for quality if not defined.
  strPathInput = config.get('input_path')
  strPathOutput = conversion['output_path'] + '/' + strFilename
  strPathOutputTmp = conversion['output_path'] + '/tmp_' + strFilename

  # open the image file
  fp = open(strPathInput + '/' + strFilename, "rb")
  image = PIL.Image.open(PIL.ImageFileIO.ImageFileIO(fp))
  image.load() # make sure we can read the raster data
  
  # Resize
  resize(image, conversion, strPathOutput)

  # Attach a annotation
  annotate(image, conversion, strPathOutput)

  # Save to a the temporary file
  # image.save(strPathOutput)
  image.save(strPathOutputTmp, quality=iQuality)
  
  # Replace the existing file if needed
  replace_file_if_changed(strPathOutput, strPathOutputTmp)
  return image.size

def resize(image, conversion, strPathOutput):
  """
    Resize the image
  """
  # Get default/configuration values
  iMaxHeight = conversion['size_max_height']
  iMaxWidth = conversion['size_max_width']

  # Calculate if resizeing is required/wanted
  iWidth, iHeight = image.size
  iOutputWidth = iMaxWidth
  iOutputHeight = iHeight*iOutputWidth/iWidth
  if iOutputHeight > iMaxHeight:
    iOutputHeight = iMaxHeight
    iOutputWidth = iWidth*iOutputHeight/iHeight
  if iOutputWidth>iWidth:
    # The image will be enlarged: This will result in bad image-quality
    # We leave the image as it was to avoid loss of quality
    print "  '%s': This image would be enlarged. Don't resize!" % (strPathOutput)
    # shutil.copyfile(strPathInput + '/' + strFilename, strPathOutput + '/tmp_' + strFilename)
    # replace_file_if_changed(strPathOutput, strFilenameTmp, strFilename)
    # return (iWidth, iHeight)
    return
  
  # Resize the image
  # image = image.resize((iOutputWidth, iOutputHeight), PIL.Image.BILINEAR)
  image = image.resize((iOutputWidth, iOutputHeight), PIL.Image.ANTIALIAS)

def replace_file_if_changed(strPathOutput, strPathOutputTmp):
  """
    Do not change timestamps if file didn't change:
    The Ouput-Image is always written in a tmp-file
    and the existing file is replaced only if changed.
  """
  if not os.path.exists(strPathOutput):
    # File didn't exist yet
    os.rename(strPathOutputTmp, strPathOutput)
    return
  if filecmp.cmp(strPathOutputTmp, strPathOutput):
    # file didn't change
    os.remove(strPathOutputTmp)
    return
  # file changed
  os.remove(strPathOutput)
  os.rename(strPathOutputTmp, strPathOutput)

def annotate(image, conversion, strPathOutput):
  """
    Write an annotation to the image
  """
  # Get prepared
  dictAnnotation = conversion.get('annotation', None)
  if dictAnnotation == None:
    # No Annotation is requried
    return
  import ImageFont, ImageDraw
  
  # Retrieve configuration/default values
  strText = dictAnnotation.get('text', 'Created by Alpuma')
  strFont = dictAnnotation.get('font', 'arial.ttf')
  iSize = dictAnnotation.get('size', 12)
  strPosition = dictAnnotation.get('position', 'bottomright')
  iSpaceingH = dictAnnotation.get('spacingh', 10)
  iSpaceingV = dictAnnotation.get('spacingv', 3)
  
  # Prepare Annoation and calculate it's size inclusive spaceing
  font = ImageFont.truetype(strFont, iSize)
  iAnnotationWidth, iAnnotationHeight = font.getsize(strText)
  iAnnotationWidth += iSpaceingH
  iAnnotationHeight += iSpaceingV
  iWidth, iHeight = image.size
  if (iAnnotationWidth > iWidth) | (iAnnotationHeight > iHeight):
    print "  '%s': Image smaller than Annotation: Don't write Annotation." % strPathOutput
    return

  # Calculate position
  if strPosition == 'topleft':
    position = (iSpaceingH, iSpaceingV)
  elif strPosition == 'bottomright':
    position = (iWidth - iAnnotationWidth, iHeight - iAnnotationHeight)
  else:
    print 'Unknown position "%s". Please correction the configurationfile!' % strPosition
    position = (iSpaceingH, iSpaceingV)
  
  # Write Annotation
  # draw.text((0, 0), strText, font=font, fill='white')
  # draw.text((0, 0), strText, font=font, fill='red')
  draw = ImageDraw.Draw(image)
  draw.text(position, strText, font=font)

if __name__=="__main__":
  go()